/* Get [USA only] National Weather Service [NWS] Zone Forecasts
For Win32 and OS/2.  R.J.Holmgren holmgren@acedsl.com 1/28/2001 LastRev.5/25/02
Required files (OS/2 only): SAA ('Classic') or Object REXX; RXSOCK.DLL
  and RXFTP.DLL appropriate to your TCP/IP syslevel
Required files (Win32 only): IBM Object Rexx for Windows
  (Interpreter Edition)

Command:
-------
[REXX.EXE] NWS.CMD stznnn  <==direct download (no lookup)
[REXX.EXE] NWS.CMD st [[n]n]n  <==direct download (no lookup)
[REXX.EXE] NWS.CMD st county|borough|parish|locality

where: st=2-character STate abbreviation
       'z'=$tring literal 'z|Z'
       n=1-3 character NWS Zone number
       county name or description of locality,
         e.g. "Coastal Hancock" [County] or "Owens Valley"
       "REXX.EXE"=Win32 only

Examples:
--------
[REXX.EXE] NWS.CMD MEZ027   <==direct access to file "mez027.txt"
[REXX.EXE] NWS.CMD me 27    <==direct access to file "mez027.txt"
[REXX.EXE] NWS.CMD Me Knox  <==lookup in zonelist ==> "mez027.txt"
[REXX.EXE] NWS.CMD ca Death Valley National Park  <==lookup
[REXX.EXE] NWS.CMD NY Manhattan  <==lookup

On the initial use requiring a lookup, file NWSZONES.TXT (69Kb) is
downloaded & filed in the directory specified in variable 'zl'
[default=".\NWSZONES.TXT"].  NWSZONES.TXT may be freely perused for
nationwide Zone information (the first 3 digits of each line in
the file constitute the pertinent NWS Zone number).  If you input
a county name or locality, NWSZONES.TXT is always consulted; if you
input a state abbreviation and zone number, lookup is unnecessary.

Some cities are named in the zonelist, others are not.

The most recent forecast fetched from NWS is filed in the
directory specified in variable 'lf' [default="FORECAST.TXT", which
files in the same d:\dir as NWS.CMD].

To view graphical maps of Zones in each state, see
http://weather.noaa.gov/pd/usframes.html

TWO (2) USER VARIABLES ([location]filenames stored on local machine): */
zl='nwszones.txt'   /* Zones List [d:\path\]filename */
lf='forecast.txt'   /* Latest Forecast [d:\path\]filename */
/* ------------------------------------------------------ */

rxutl=RxFuncQuery("SysLoadFuncs")
if rxutl then do
  call RxFuncAdd "SysLoadFuncs","RexxUtil","SysLoadFuncs"
  call SysLoadFuncs
  end
rxftp=0
if pos('\',lf)>0 then do
  if length(lf)>1 then sdir=left(lf,lastpos('\',lf))
  else sdir=left(directory(),2)||'\'
  end
else do
  parse source . . sdir
  sdir=left(sdir,lastpos('\',sdir))
  end
if pos('\',lf)=0 then lf=sdir||lf
if pos('\',zl)=0 then zl=sdir||zl
parse arg hf
hf=strip(hf)
if length(hf)=0 then call Help
hf=translate(hf,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')
st=left(hf,2)
if pos(st||'|','ak|al|ar|az|ca|co|ct|dc|de|fl|ga|gu|hi|ia|id|il|in|ks|ky|la|ma|md|me|mi|mn|mo|mp|ms|mt|nc|nd|ne|nh|nj|nm|nv|ny|oh|ok|or|pa|pr|ri|sc|sd|tn|tx|ut|va|vi|vt|wa|wi|wv|wy|')>0 then do
  zone=''
  if length(hf)<7&length(hf)>3 then do
    zone=right(hf,length(hf)-3)
    if pos(' ',zone)>0 then zone=right(zone,3-pos(' ',zone))
    if verify(zone,'0123456789')\=0 then zone=''
    end
  if zone='' then do
    zone=strip(substr(hf,3))
    if length(zone)>0 then do
      r=stream(zl,'C','query exists')
      if length(r)=0 then do
        rxsck=RxFuncQuery("SockLoadFuncs")
        if rxsck then do
          call RxFuncAdd 'SockLoadFuncs','RxSock','SockLoadFuncs'
          call SockLoadFuncs
          call SysCls
          end
        socket=SockSocket('AF_INET','SOCK_STREAM',0)
        if socket>-1 then do
          rc=SockGetHostByName('www.serve.com','host.!')
          host.!family='AF_INET'
          host.!port=80
          say 'Fetching "http://www.serve.com/xywwweb/nwszones.txt"'
          rc=SockConnect(socket,'host.!')
          if rc=0 then do
            crlf ='0D0A0D0A'x
            rc=SockSend(socket,'GET /xywwweb/nwszones.txt HTTP/1.0'||crlf)
            if rc>-1 then do
              do until pos(crlf,sockdata)>0
                rc=SockRecv(socket,'sockdata',1024)
                end
              call stream zl,'C','open write'
              sockdata=substr(sockdata,pos(crlf,sockdata)+4)
              call charout zl,sockdata
              do until rc=0
                rc=SockRecv(socket,'sockdata',1024)
                call charout zl,sockdata
                end
              call stream zl,'C','close'
              end
            end
          end
        call SockShutDown socket,2
        call SockClose socket
        if rxsck then call SockDropFuncs
        end
      call stream zl,'C','open read'
      do until x=st
        x=linein(zl)
        end
      do until pos(translate(zone),y)>0|length(x)=2
        x=linein(zl)
        y=translate(x)
        end
      if length(x)=2 then do
        say 'No locality "'zone'" found in 'translate(st)'; Aborting...'
        call bye
        end
      call stream zl,'C','close'
      say st||'z'||x
      say
      zone=left(x,3)
      end
    else do
      say 'No county|locality specified; Aborting...'
      call bye
      end
    end
  do while length(zone)<3
    zone='0'||zone
    end
  hf=st||'z'||zone||'.txt'
  if stream(lf,'C','query exists')\='' then call SysFileDelete lf
  targdir='data/forecasts/zone/'||st||'/'
  rxftp=RxFuncQuery("FtpLoadFuncs")
  if rxftp then do
    call RxFuncAdd "FtpLoadFuncs","rxFtp","FtpLoadFuncs"
    call FtpLoadFuncs
    end
  if FtpSetUser('weather.noaa.gov','anonymous','anonymous') then do
    if FtpChDir(targdir)=0 then do
      if FtpGet(lf,hf,'Ascii')=0 then do
        call SysCls
        y=0
        do while lines(lf)<>0
          x=linein(lf)
          if y=0&pos('.',x)=1 then do
            say
            y=1
            end
          if length(x)>1 then say x
          x=SysCurPos()
          if left(x,pos(' ',x)-1)>=22 then do
            call Charout,'Hit any key to continue...'
            call SysGetKey 'noecho'
            call SysCls
            end
          end
        say
        say 'Fetched "weather.noaa.gov/'||targdir||hf||'"'
        say 'Filed as "'||lf||'"'
        end
      else say 'Invalid host filename "'||hf||'"'
      end
    else say 'Invalid state "'||st||'"'
    call FtpLogoff
    end
  else say 'Invalid or unavailable host "weather.noaa.gov"'
  end
else say 'Invalid state "'||st||'"'
bye:
if rxftp then call FtpDropFuncs
call Charout,'Hit any key to quit...'
call SysGetKey 'noecho'
say
if rxutl then call SysDropFuncs
exit
return

HELP:
  x=0;if pos('Window',SysVersion())=1 then x=1
  say 'NWS.CMD v1.0 for SAA|Object REXX  R.J.Holmgren 1/28/2001'
  say 'Display National Weather Service [NWS] Zone Forecasts'
  say
  say 'Command:'
  say '-------'
  if x then call Charout,'REXX.EXE '
  say 'NWS.CMD stznnn  (st=STate  z="z"  [[n]n]n=ZoneNumber)'
  if x then call Charout,'REXX.EXE '
  say 'NWS.CMD st [[n]n]n'
  if x then call Charout,'REXX.EXE '
  say 'NWS.CMD st county|borough|parish|locality'
  say '  See NWS.CMD for more info & examples'
  say
  call Bye
RETURN

